
from Tkinter import Tk, Canvas, Button, Label
from ClickedFunction import *

class JetWidgetProto:
    def __init__(self, name, type):
        self.name = name
        self.type = type

class JetWidget:
    """Widget for JetScript's canvas.

A Widget has a graphical representation that can be interacted with, and some
data about its actual state (a type attribute and a dictionary of attributes).

Connections between Widgets are kept elsewhere.

We renamed this JetWidget so that there weren't name conflicts with the Tkinter
Widget class.

To create a new Widget, try:
w = JetWidget(canvas, "name")
    """
    #There are two major parts to the JetWidget:
    # the gui stuff
    # the script-generation stuff
    MULTCONNECTASPECTS = ["cli", "stdin"]
    def __init__(self, graph, command, x = 0, y = 0, attrs=""):
        self.loc = [x, y]
        self.command = command
        #self.attribs might be more useful as a dictionary
        self.attribs = attrs
        self.graph = graph
        self.myIndex = 0
        self.create_components()
        #Tag is generated to refer only to this widget. I'd use self.id but
        # numeric tags aren't allowed.
        #alltag represents all components of the JetWidget -- label, rectangle,
        #button, whatever. maintag is just the label, rectangle and button.
        #Other tags will refer to other parts of the button -- for instance,
        #rectangles to represent stdin, stdout, sterr, CLI?
        #The rectangle must be below the text
        graph.lower(self.id)
        self.__init_bindings()
        self.lastmouseloc = []
        self.connectsdict = {} # This is an adjacency list
    def CanAddConnection(self, aspect):
        if aspect in self.MULTCONNECTASPECTS:
            #Always can add another argument or another stdin
            return True
        if self.connectsdict.has_key(aspect):
            return False
        return True
        
    def AddConnection(self, connection, aspect):
        if self.CanAddConnection(aspect) == False:
            #Something needs to happen here to get rid of the other connection
            # For now just fail
            return False
        if aspect in self.MULTCONNECTASPECTS:
            if self.connectsdict.has_key(aspect):
                self.connectsdict[aspect].append(connection)
            else:
                self.connectsdict[aspect] = [connection]
        else:
            self.connectsdict[aspect] = connection
        return True
    def HasConnection(self, aspect):
        return self.connectsdict.has_key(aspect)
    def GetConnection(self, aspect):
        return self.connectsdict[aspect]
    def DelConnection(self, aspect):
        if self.HasConnection(aspect):
            del self.connectsdict[aspect]

    #There's currently three types of exploration.
    #One explores a pipeline, such as:
    # a | b | c
    #going from stdout -> stdin and so forth, so if something is
    #connected to this one's stdin, it's a parent. The pipeline is explored
    #from a, to b, to c -- therefore c's parent is b, b's parent is a.
    def Parent(self):
        """Returns the parent of this node in this pipeline, if there is one.
        If there is more than one parent, they are returned as a list.
        Otherwise returns None."""
        if self.connectsdict.has_key("stdin"):
            return [c.OtherWidget(self) for c in self.connectsdict["stdin"]]
        return None

        
    #The second type of exploration explores a set of pipelines.
    #Imagine:
    #  a | b `c | d` | e
    # We would like the widgets to be explored in alphabetical order.
    #This can be tricky, because the stdout of *d* is connected to cli of
    #b.
    #How we do this is first, we find the head of the TotalPipeline, which
    #is in this case a, and exploring the pipeline, which means going to
    #b via stdout. Then, after visiting b, we recursively explore the
    #`c|d` pipeline, which means starting at d, and going up till we find
    #the root of *that* pipeline, which is c, and exploring from there.
        
    #If it looked like this:
    # a | b | c `d | e \`f | g\`` | h
    #The recursive-exploration of the commands connected to cli would take
    # care of the double-cli connection.
    def PipeSetParent(self):
        """Returns the parent of this node, in any pipeline, if there is one.
        Otherwise returns None."""
        if self.Parent():
            return self.Parent()
        for c in ["stdout", "stderr"]:
            if self.connectsdict.has_key(c) and\
                   self.connectsdict[c].ConnectsToAspect("cli"):
                return self.connectsdict[c].OtherWidget(self)
        return None
    

    #Third type of exploration takes into account the flow of control.
    def CtrlSetParent(self):
        if self.connectsdict.has_key("ctrli"):
            return self.connectsdict["ctrli"]



    #Any of the above Parent() functions can return lists!
    #As a result, note the following convenience functions:
    def AcceptEach(self, func, p, visitor):
        visitor.AddParen()
        visitor.PushStoppingPoint(self)
        func(p[0], visitor)
        for w in p[1:]:
            visitor.AddSemi()
            visitor.PushStoppingPoint(self)
            func(w, visitor)
        visitor.SubtParen()
            
    #Corresponding to each type of exploration, there's a recursive
    # exploration function that needs to travel to the "root" of the
    # exploration. The Accept functions serve that purpose. They all need
    # to recurse according to whether their parents exist, or if
    # they have numerous parents. Thus this helper function.
    def RecurseMaybeParents(self, func, p, visitor):
        """Recurses according to whether a parent exists (supplied as
        second argument). If it does, call func on each parent.
        If numerous parents, tell the visitor to stop exploration at this
        node."""
        print "Recurse parents"
        if p or visitor.LoopDetected(self):
            print "recursing on p"
            print p
            visitor.setLooped(self)
            if type(p) == list:
                self.AcceptEach(func, p, visitor)
                #OK, resume exploration with this widget:
                self.Accept(visitor)
            else:
                #Hopefully there's just one parent and it's a JetWidget
                func(p, visitor)
        else:
            self.Accept(visitor)

    def AcceptPipeline(self, visitor):
        p = self.Parent()
        self.AcceptMaybeParents(JetWidget.AcceptPipeline, p, visitor)

    def AcceptPipeSetPipeline(self, visitor):
        p = self.PipeSetParent()
        self.RecurseMaybeParents(JetWidget.AcceptPipeSetPipeline, p, visitor)

    def Accept(self, visitor):
        if visitor.VisitedMe(self) == True:
            return
        visitor.Visit(self)
        if self.connectsdict.has_key("cli"):
            for c in self.connectsdict["cli"]:
                visitor.AddCLI()
                c.OtherWidget(self).AcceptPipeline(visitor)
                visitor.SubtCLI()
        if self.connectsdict.has_key("stdout") and\
               self.connectsdict["stdout"].ConnectsToAspect("stdin"):
            visitor.OutputPipe()
            self.connectsdict["stdout"].OtherWidget(self).Accept(visitor)
        if self.connectsdict.has_key("stderr"):
            fd = visitor.OutputToFD()
            

    def create_components(self):
        [x, y] = self.loc
        self.labelid = self.graph.create_text(x + 5, y + 5, text = self.command, \
                                         anchor = "nw")
        labelsize = self.graph.bbox(self.labelid)
        self.id = self.graph.create_rectangle(x, y, labelsize[2] + 5,
                                          labelsize[3]+5, fill = "light blue")
        rectwidth = labelsize[2] - labelsize[0] + 10
        rectheight = labelsize[3] - labelsize[1] + 10
        print rectheight, rectwidth
        print x, labelsize[2]+5
        from ConnectionPoint import ConnectionPoint
        self.alltag = self.command+str(self.id)
        self.maintag = self.command+"m"+str(self.id)
        #print "Creating connectionpoint: self = %s" %(self,)
        self.stdin = ConnectionPoint(self, self.graph, [x, y-5],
                                     [x+(rectwidth/2), y], "stdin",self.alltag)
        self.cli = ConnectionPoint(self, self.graph,
                                      [x+(rectwidth/2), y-5],
                                     [x+rectwidth-1, y], "cli",self.alltag)
        self.stdout = ConnectionPoint(self, self.graph, [x, y+rectheight],
                                     [x+(rectwidth/2), y+rectheight+5],
                                      "stdout",self.alltag)
        self.stderr = ConnectionPoint(self, self.graph,
                                      [x+(rectwidth/2), y+rectheight],
                                      [x+rectwidth-1, y+rectheight+5],
                                      "stderr",self.alltag)
        self.ctrli = ConnectionPoint(self, self.graph, [x-5,y],
                                     [x, y+rectheight], "ctrli", self.alltag)
        #I don't think a dictionary will be necessary -- that's why
        # Python has getattr. Will need to worry about this when loading
        #  etc.
        #note - Enabled dictionary for loading
        self.aspectdict = {"stdout":self.stdout, "stderr":self.stderr,
                           "cli":self.cli, "stdin":self.stdin, "ctrli":self.ctrli}
        [self.graph.addtag_withtag(self.alltag, wid) for wid in
         [self.labelid, self.id]]
        [self.graph.addtag_withtag(self.maintag, wid) for wid in
         [self.labelid, self.id]]

    def ClickedAspect(self, widget, label, event, on):
        print "Got %sclicked on my %s" % ((on and [""] or ["un"])[0], label)
        self.graph.ClickedWidgetAspect(self, widget, label, event, on)

    def DisconnectAspect(self,label,on):
        if on:
            pass
        else:
            if self.HasConnection(label):
                self.graph.DisconnectAspect(self.connectsdict[label])
                self.connectsdict[label].Destroy()
    def MovedAspect(self, aspect, label, event):
        self.graph.MovedWidgetAspect(self, aspect, label, event)

    def Clicked(self, event, on):
        self.graph.ClickedWidget(self, self.alltag, event, on)

    def __init_bindings(self):
        #button 1:
        Connect(self.graph, self.maintag, 1, self, JetWidget.Clicked)
        self.graph.tag_bind(self.maintag, "<Motion>", self.__move)
    def __move(self, event):
        #This will not be forwarded up to graph, because the widget can move
        #all by itself. -- no, scratch that --
        #Graph may need to know if there is motion, because there may need to
        #be a connection that needs to be drawn..
        self.graph.MovedWidget(self, self.alltag, event)
        self.graph.MovingLoc(event.x,event.y,self.command,self.attribs)
    def AttribsString(self):
        s = ""
        for (k, v) in self.attribs.items():
            s += "%s %s" % (k, v)
        return s
    def AddArguments(self, args):
        self.attribs=args
        #"""Add arguments as given by args. Form is "%s %s"."""
        #argl = args.split(" ")
        #print "argl = %s" %(argl,)
        # So, for instance, now argl = ["-C", "2", "-i"]
        # len(argl) = 3. range(0, len(argl)-1, 2) = [0]
        # So associate -C with 2, and -i by itself (at the end).
        # More effective techniques should be employed here, for instance a
        # context-sensitive GUI menu, where you could enable/disable options.
        #for i in range(0, len(argl)-1, 2):
         #   print "Adding %s => %s" %(argl[i], argl[i+1])
          #  self.attribs[argl[i]] = argl[i+1]
        #if len(argl) % 2 == 1:
         #   self.attribs[argl[len(argl)-1]] = ""
    def moveby(self,delta):
        print "Not doing anything here, because it's unnecessary!!"
        #self.loc = map(lambda old,d:old+d, self.loc, delta)
    def moveto(self,loc):
        self.graph.move(self.alltag, loc[0] - self.loc[0],
                        loc[1] - self.loc[1])
        self.loc = loc
    def __repr__(self):
        return str(self.id) + ", " + str(self.loc) + ", " +\
               str(self.command) + ", " + str(self.attribs)
    def setIndex(self,i):
        self.myIndex=int(i)
    def Destroy(self):
        """destroy this object, including removing from the graph"""
        self.graph.delete(self.labelid)
        self.graph.delete(self.id)
        for w in [self.stdin, self.stdout, self.stderr, self.cli, self.ctrli]:
            w.Destroy()
        for value in self.connectsdict.values():
            value.Destroy()

if __name__ == "__main__":
    from Graph import Graph
    from ScrollList import WidgetPickList
    picklist = WidgetPickList(Tk(),lambda a: None)
    root = Tk()
    graph = Graph(root, picklist)
    graph.pack()
    w = JetWidget(graph)
    print w
    v = JetWidget(graph, 10, 100, "cat")
    print v
    x = JetWidget(graph, 100, 100, "cat",
                  attrs = {"filename" : "JetScript.py"})
    print x
    from time import sleep
    sleep(1)
    #x.moveby([110, 156])
    root.mainloop()
