from Tkinter import *
from JetWidget import *
import string

 
class WidgetPickList ( Frame ):

    WIDGET_NAMES_FILE = "WidgetList.txt"       # Widget listing file
    NAME_WIDTH = 20     # Width of the Listbox in characters
    NAME_LINES = 22     # Number of lines in the Listbox

    def __init__ ( self, master, callback ):
        Frame.__init__ ( self, master )
        self._callback  =  callback
        self._createWidgets ()

    def _createWidgets ( self ):
        self._scroll  =  Scrollbar ( self, orient=VERTICAL )
        self._scroll.grid ( row=0, column=1, sticky=N+E+S )
        self._names  =  Listbox ( self, width = self.NAME_WIDTH,
            height = self.NAME_LINES, relief = SUNKEN,
            borderwidth = 2, exportselection = 0,
            yscrollcommand = self._scroll.set )
        self._names.grid ( row=0, column=0, sticky=N+W+S+E )
        self._names.bind ( "<Double-Button-1>", self._pickHandler )
        self._scroll["command"] = self._names.yview
        self.grid(row = 1, column = 1)
        self._readNamedWidgets ( )
        #self._createConnectionWidgets()
        self._createSeqWidget()


    def _pickHandler ( self, event ):
        #Note currently unused. might want to kill it before beta release
        """
            if
            event is an event in self._names
            then
            call self._callback(i)where i is the index of the selected name
        """
        current = self._names.curselection()    # An Index tuple
        if  len(current) > 0:                   # Ignore when empty
            i   =  string.atoi ( current[0] )
            self._callback ( i )

    def clear(self):
        "Clear the selected widget, so that no widget is selected."
        self._names.selection_clear(0, self._names.size())

    def currentWidget(self):
        "Widget the picklist is currently pointing to."
        current = self._names.curselection()
        if len(current) > 0:
            return self._widgetlist[int(current[0])]
        else: return None

    def _readNamedWidgets ( self ):
        """
            read widget names and ignore lines starting with !
        """
        try:
            widgetFile = open ( self.WIDGET_NAMES_FILE )
        except:
            print ( "*** Can't open the widget names file `" +
                    self.WIDGET_NAMES_FILE + "'." )
            print ( "*** Please make sure file is in the program directory")
            return

        widgetList  =  widgetFile.readlines()
        widgetFile.close()
        
        self._widgetlist=[]
        for line in widgetList:
             if line[0] != "!":  # Comment lines are delimted by a '!'
                 self._names.insert ( END, string.strip( line ) ) #strip \n and \r
                 self._widgetlist.append(JetWidgetProto(string.strip(line),
                                                       JetWidget))

    def _insertCustomWidget(self):
        import tkSimpleDialog
        name = tkSimpleDialog.askstring("custom widget","Enter name of new command")
        for item in self._widgetlist:
            if name == item.name:
                return "failed"
        self._names.insert(END, name)
        self._widgetlist.append(JetWidgetProto(name,JetWidget))

    def _createConnectionWidgets(self):
        #function no currently used. preempted by auto left/right clicking
        from Connection import Connection
        self._names.insert(END, "<-connect->")
        self._widgetlist.append(JetWidgetProto("connection", Connection))

    def _createSeqWidget(self):
        from SeqWidget import SeqWidget
        self._names.insert(END, "Sequencer")
        self._widgetlist.append(JetWidgetProto("Sequencer", SeqWidget))
